#include <stdint.h>

// Note: compiles correctly with icc, but not with gcc 4.8.2/Ubuntu. Supposedly
// the bug was supposed to be fixed.

void
dotproduct_omp(double * sumref, 
               double * A, 
               double * B, 
               const uint64_t  N)
{
    double sum;
    
    sum = 0;
    #pragma omp parallel for default(none) shared(A,B) reduction(+:sum)
    for (uint64_t i = 0; i < N; ++i) {
        sum += A[i] * B[i];
    }
    *sumref = sum;
}

void
dotproduct_omp_critical(double * sumref, 
               double * A, 
               double * B, 
               const unsigned long  N)
{
    double sum = 0.0;
    
    #pragma omp parallel for default(none) shared(A,B,sum)
    for (uint64_t i = 0; i < N; ++i) {
        #pragma omp critical
        {
            sum += A[i] * B[i];
        }
    }
    *sumref = sum;
}

void
dotproduct_omp_atomic(double * sumref, 
               double * A, 
               double * B, 
               const unsigned long  N)
{
    double sum = 0.0;
    
    #pragma omp parallel for default(none) shared(A,B,sum)
    for (uint64_t i = 0; i < N; ++i) {
        #pragma omp atomic
        sum += A[i] * B[i];
    }
    *sumref = sum;
}
