#include "dotproduct.h"

void *
dotproduct_task_atomic(void *arg)
{
    dotproduct_atomic_t  *dp  = (dotproduct_atomic_t *)arg;
    double                sum = 0.0;
    uint64_t              lo  = (dp->id     * dp->N) / dp->nthreads;
    uint64_t              hi  = ((dp->id+1) * dp->N) / dp->nthreads;

    for (uint64_t i = lo; i < hi; ++i) 
        sum += dp->A[i] * dp->B[i];

    (void) atomic_add_dbl(dp->sum,sum);

    return NULL; /* no return state to give here */
}

void 
dotproduct_pthread_atomic(double *sumref, double* A, double* B, 
                          const uint64_t N, const uint64_t n_threads)
{
    pthread_t           threads[n_threads];
    dotproduct_atomic_t dp[n_threads];
    volatile double sum = 0.0;
    pthread_attr_t attr;
    pthread_attr_init(&attr);
    /* pedantic: default value is already PTHREAD_CREATE_JOINABLE */
    pthread_attr_setdetachstate(&attr,PTHREAD_CREATE_JOINABLE); 
    for (uint64_t i = 0; i < n_threads; ++i) {
        dp[i] = (dotproduct_atomic_t){ 
            .A=A, .B=B, .sum=&sum, .N=N, .id=i, .nthreads=n_threads 
        };
        spthread_create(&threads[i],&attr, dotproduct_task_atomic, &dp[i]);

    }
    for (uint64_t i = 0; i < n_threads; ++i)
        spthread_join(threads[i],NULL);

    *sumref = sum;

    return;
}
