#include "utils.h"

void dotproduct_pthread_atomic(double *sumref, double * A, double * B, 
                const long  N, const long  nb_thread);
void dotproduct_pthread(double *sumref, double * A, double * B, 
                const long  N, const long  nb_thread);

#ifndef NUMTHREADS 
#define NUMTHREADS 4UL
#endif

#ifndef N
#define N 100L
#endif

void
dotproduct_seq(double *sumref, double *A, double *B, const uint64_t UPPER)
{
    double sum = 0.0;
    for (uint64_t i = 0; i < UPPER; ++i)
        sum += A[i] * B[i];
    *sumref = sum;
}

int
main(void)
{
    double *A         = smalloc(sizeof(double) * N), 
           *B         = smalloc(sizeof(double) * N);
    long    n         = N;
    long    nbthreads = NUMTHREADS;
    double  sum       = 0.0;

    for (long i = 0; i < n; ++i) {
        A[i] = i;
        B[i] = n-i;
    }

    sum = 0.0;
    dotproduct_seq(&sum,A,B,n);
    printf("Sequential dot product                     = %-16.2f\n", sum);

    sum = 0.0;
    dotproduct_pthread(&sum,A,B,n,nbthreads);
    printf("Parallel (PTHREAD) dot product -- critical = %-16.2f\n", sum);

    sum = 0.0;
    dotproduct_pthread_atomic(&sum,A,B,n,nbthreads);
    printf("Parallel (PTHREAD) dot product -- atomic   = %-16.2f\n", sum);

    free(A);
    free(B);

    return 0;
}
