/*
 * Copyright (c) 2014, Stéphane Zuckerman
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef LINKED_LIST_H
#define LINKED_LIST_H

#include <stdint.h>
#include <stddef.h>

/**
 * A link in a linked list -- data is symbolized by an int
 */
typedef struct link_s {
    struct link_s *next;
    int            data;
} link_t;


/**
 * Linked List Abstract Data Type
 */
typedef struct linked_list_s {
    link_t   *head,
             *tail;
    size_t    size;
    link_t* (*new_link)     (int);
    void    (*destroy_link) (link_t*, int*);
    void    (*push_back)    (struct linked_list_s*, int);
    void    (*push_front)   (struct linked_list_s*, int);
    link_t* (*find)         (struct linked_list_s* list, link_t* start, const int value);

} linked_list_t;

void list_init (struct linked_list_s*, 
                link_t* (*new_link) (int), 
                void (*destroy_link) (link_t*, int*),
                void (*push_back)  (linked_list_t*,int), 
                void (*push_front) (linked_list_t*,int),
                link_t* (*find) (linked_list_t*,link_t*,const int));

link_t*   link_init    (int);
void      link_destroy (link_t*,int*);
void      list_destroy (linked_list_t*);
void      push_back    (linked_list_t*,int);
void      push_front   (linked_list_t*,int);
link_t* find           (linked_list_t*,link_t*,const int);
void remove_after(linked_list_t* list, link_t* l, int* value);
void insert_after(linked_list_t* list, link_t* l, int  value);

void process_list(linked_list_t*,void (*processor)(link_t*));

#endif // LINKED_LIST_H
