/*
 * Copyright (c) 2014, Stéphane Zuckerman
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <stdio.h>
#include <stdlib.h>
#include "linked_list.h"

const long DEFAULT_MAX = 10L;

void
double_value(link_t* link) 
{
    link->data *= 2;
}

void
sum_until(link_t* link)
{
    int sum = 0;
    for (int i = 0; i < link->data; ++i)
        sum += i;
    link->data = sum;
}

int
main(int argc, char *argv[])
{
    srand(1UL);

    long n_gend_ints = DEFAULT_MAX;
    if (argc == 2) {
        char* strerr = NULL;
        n_gend_ints  = strtol(argv[1],&strerr,0);
        if (*strerr != '\0' || n_gend_ints < 0) {
            printf("USAGE:\t%s [n, n=%ld by default]\n", argv[0], DEFAULT_MAX);
            exit(EXIT_SUCCESS);
        }
    }

    linked_list_t list;
    list_init(&list,link_init,link_destroy,push_back,push_front,find);

    printf("Generating %ld random integers)\n", n_gend_ints);
    for (int i = 1; i <= n_gend_ints; ++i) 
        insert_after(&list, NULL, rand() % i);

    puts("Before processing:");
    for (link_t* cur = list.head; cur != NULL; cur = cur->next) 
        printf("link value = %d\n", cur->data);

    puts("processing");
    process_list(&list, sum_until);
    process_list(&list, double_value);

    puts("After processing:");
    for (link_t* cur = list.head; cur != NULL; cur = cur->next) 
        printf("link value = %d\n", cur->data);

    list_destroy(&list);

    printf("The list is %sempty\n", list.head == NULL ? "" : "not ");
    return 0;
}
