/*
 * Copyright (c) 2014, Stéphane Zuckerman
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include "linked_list.h"
#include "utils.h"
#include "linked_list_pthd.h" //<pthread_wrappers.h>
#include <stdbool.h>

static inline pthread_t
spawn(void* (*code)(void*), void* data)
{
    pthread_t thread;

    pthread_attr_t attr;
    pthread_attr_init(&attr);
    /* pedantic: default value is already PTHREAD_CREATE_JOINABLE */
    pthread_attr_setdetachstate(&attr,PTHREAD_CREATE_JOINABLE); 

    spthread_create(&thread,&attr,code,data);

    return thread;
}

static inline void
sync(pthread_t thread)
{
    void* freeme = NULL;
    spthread_join(thread,&freeme);
    if (freeme)
        free(freeme);
}

typedef struct processor_s {
    void   (*processor)(link_t*);
    link_t  *link;
//  volatile uint64_t *done;
} processor_t;

void*
process_link(void* frame)
{
    processor_t* p = (processor_t*) frame;
    p->processor(p->link);
//  if (p->link->next == NULL)
//      while(__sync_bool_compare_and_swap(p->done,0,1) == false)
//          ;
    return p;
}

void
process_list(linked_list_t* list, void (*processor)(link_t*))
{
    link_t*           current      = list->head;

    pthd_linked_list_t pthd_list;
    pthd_list_init(&pthd_list,pthd_link_init,pthd_link_destroy,pthd_push_back,pthd_push_front,pthd_find);
    
    while (current != list->tail) {
        processor_t* frame = smalloc(sizeof(processor_t));
        *frame             = (processor_t){ .processor = processor, .link = current };
        pthd_list.push_back(&pthd_list,spawn(process_link,frame));
        current            = current->next;
    }

    for (pthd_link_t* cur = pthd_list.head; cur != NULL; cur = cur->next) 
        sync(cur->id);
    
    pthd_list_destroy(&pthd_list);
}
